-- This Source Code Form is subject to the terms of the bCDDL, v. 1.1.
-- If a copy of the bCDDL was not distributed with this
-- file, You can obtain one at http://beamng.com/bCDDL-1.1.txt

local M = {}
M.type = "auxiliary"
M.relevantDevice = nil

local htmlTexture = require("htmlTexture")

local min = math.min
local max = math.max

local gaugesScreenName = nil
local htmlPath = nil

local isAnalog = false

local updateTimer = 0
local updateFPS = 15
local invFPS = 1 / updateFPS

local volts = 0
local voltRampUpRate = 14*2

local function setUpdateFps(fps)
  updateFPS = fps
  invFPS = 1 / updateFPS
end

local function getIsEngineRunning()
  return (electrics.values["engineRunning"] or 0) > 0.5
end

local function isNaN( v ) return type( v ) == "number" and v ~= v end 

local function updateGFX(dt) 
	--screen update
	updateTimer = updateTimer + dt
	if updateTimer > invFPS then
		-- set mileage or error text
		if isNaN(electrics.values["wheelspeed"] or 0) then
			if isAnalog then
				htmlTexture.call(gaugesScreenName, "setTextFromLua", {value=0})
			else
				htmlTexture.call(gaugesScreenName, "setTextFromLua", {value="service"})
			end
		else
			local mileage = odo.getValue() 
			htmlTexture.call(gaugesScreenName, "setMileageFromLua", {value=mileage})
		end

	end
end

local function init(jbeamData)
	gaugesScreenName = jbeamData.materialName
	htmlPath = jbeamData.htmlPath
	local unitType = jbeamData.unitType or "metric"
	local width = jbeamData.textureWidth or 280
	local height = jbeamData.textureHeight or 132
	isAnalog = jbeamData.isAnalog or false
	setUpdateFps(jbeamData.updateFps or 15)


	htmlTexture.create(gaugesScreenName, htmlPath, width, height, updateFPS, "automatic")
	htmlTexture.call(gaugesScreenName, "init", {})
	htmlTexture.call(gaugesScreenName, "setUnitTypeFromLua", {unitType = unitType})
	htmlTexture.call(gaugesScreenName, "setMileageFromLua", {value=0})
end

M.init = init
M.updateGFX = updateGFX

return M
